@@ -151,7 +151,7 @@ template<typename C_t> [[nodiscard]] C_t PlWrap(C_t rc, qid_t qid = 0);
 // This is for functions that report errors but don't have an
 // indication of "fail" - that is, almost everything except for
 // functions like PL_unify_*() or PL_next_solution().
-template<typename C_T> void PlEx(C_T rc, qid_t qid = 0);
+template<typename C_t> void PlEx(C_t rc, qid_t qid = 0);
 
 // Check the return code: if "false", throw PlFail.
 inline void PlCheckFail(bool rc);
@@ -483,6 +483,8 @@ public:
 
   [[nodiscard]] bool get_file_name(char **name, int flags) const { return Plx_get_file_name(unwrap(), name, flags); }
   [[nodiscard]] bool get_file_nameW(wchar_t **name, int flags) const { return Plx_get_file_nameW(unwrap(), name, flags); }
+  [[nodiscard]] const std::string get_file_name(int flags) const;
+  [[nodiscard]] const std::wstring get_file_nameW(int flags) const;
 
   [[nodiscard]] bool get_attr(term_t a) const { return Plx_get_attr(unwrap(), a); }
 
@@ -746,7 +748,7 @@ public:
 
   void reset_term_refs() { Plx_reset_term_refs(unwrap()); }
 
-  int call(PlModule module = PlModule()) const { return Plx_call(unwrap(), module.unwrap()); }
+  bool call(PlModule module = PlModule()) const { return Plx_call(unwrap(), module.unwrap()); }
 
 private:
   bool eq(const char *s) const;
@@ -941,9 +943,12 @@ public:
   explicit PlPredicate(PlFunctor f, PlModule m)
     : WrappedC<predicate_t>(Plx_pred(f.unwrap(), m.unwrap()))
   { }
-  explicit PlPredicate(const char *name, int arity,  const char *module)
+  explicit PlPredicate(const char *name, int arity, const char *module)
     : WrappedC<predicate_t>(Plx_predicate(name, arity,  module))
   { }
+  explicit PlPredicate(const std::string& name, int arity, const std::string& module)
+    : WrappedC<predicate_t>(Plx_predicate(name.c_str(), arity,  module.c_str()))
+  { }
   void predicate_info(PlAtom *name, size_t *arity, PlModule *module)
   { atom_t n;
     module_t m;
@@ -1267,19 +1272,19 @@ protected:
 
 PlException PlGeneralError(PlTerm inside);
 
-PlException PlTypeError(const std::string& expected, PlTerm actual);
+PlException PlTypeError(const std::string& expected, PlTerm culprit);
 
-PlException PlDomainError(const std::string& expected, PlTerm actual);
+PlException PlDomainError(const std::string& expected, PlTerm culprit);
 
-PlException PlDomainError(PlTerm expected, PlTerm actual);
+PlException PlDomainError(PlTerm expected, PlTerm culprit);
 
-PlException PlInstantiationError(PlTerm t);
+PlException PlInstantiationError(PlTerm culprit);
 
-PlException PlUninstantiationError(PlTerm t);
+PlException PlUninstantiationError(PlTerm culprit);
 
 PlException PlRepresentationError(const std::string& resource);
 
-PlException PlExistenceError(const std::string& type, PlTerm actual);
+PlException PlExistenceError(const std::string& type, PlTerm culprit);
 
 PlException PlPermissionError(const std::string& op, const std::string& type, PlTerm obj);
 
@@ -1890,7 +1895,7 @@ public:
   static void acquire(atom_t a)
   { PlAtom a_(a);
     auto data = cast_check(a_);
-    bool rc;
+    bool rc; // Uninitialized variable warning (some compilers)
     try
     { data->acquire(a_);
       rc = true;
@@ -1923,8 +1928,8 @@ public:
     // types - they should have been already compared by standard
     // order of types; but use cast_check() anyway (which will be
     // optimised away if NDEBUG).
-    bool rc_try = false;
-    int rc;
+    bool rc_try = false; // Uninitialized variable warning (some compilers)
+    int rc = 0;          // Uninitialized variable warning (some compilers)
     try
     { const auto a_data = cast(PlAtom(a));
       const auto b_data = cast(PlAtom(b));
@@ -1945,7 +1950,7 @@ public:
     if ( !data )
       // TODO: demangle typeid::name()
       return Sfprintf(s, "<%s>(%p)", typeid(C_t).name(), data) >= 0;
-    int rc;
+    int rc = -1; // Uninitialized variable warning (some compilers)
     try
     { rc = data->write(s, flags);
     }
@@ -1959,7 +1964,7 @@ public:
   { const auto data = cast(PlAtom(a));
     if ( !data )
       return false;
-    bool rc;
+    bool rc = false; // Uninitialized variable warning (some compilers)
     try
     { data->save(fd);
       rc = true;
@@ -1973,7 +1978,7 @@ public:
   static atom_t load(IOSTREAM *fd)
   { C_t ref;
     atom_t atom;
-    int rc_try;
+    int rc_try = false; // Uninitialized variable warning (some compilers)
     try
     { atom = ref.load(fd).unwrap();
       rc_try = true;
