@@ -51,12 +51,22 @@
 
 :- use_foreign_library(foreign(test_cpp)).
 
+:- dynamic user:file_search_path/2.
+:- multifile user:file_search_path/2.
+
+:- prolog_load_context(directory, Dir),
+    asserta(user:file_search_path(my_program_home, Dir)).
+
 :- multifile user:portray/1.
 
 user:portray(MyBlob) :-
     blob(MyBlob, my_blob), !,
     portray_my_blob(current_output, MyBlob).
 
+user:portray(MyFileBlob) :-
+    blob(MyFileBlob, my_file_blob), !,
+    my_file_blob_portray(current_output, MyFileBlob).
+
 % test_cpp :-
 %     run_tests([ cpp,
 %                 cpp_atommap,
@@ -924,6 +934,45 @@ test(blob_portray, S == "MyBlob(closed)") :-
     close_my_blob(B),
     with_output_to(string(S), print(B)).
 
+expected_file_name_my_program_home(ShortPath, AbsPathOS) :-
+    once(user:file_search_path(my_program_home, Home)),
+    concat_atom([Home, '/', ShortPath], AbsPath0),
+    prolog_to_os_filename(AbsPath0, AbsPathOS).
+
+expected_file_name_my_program_home_string(ShortPath, AbsPathOS) :-
+    expected_file_name_my_program_home(ShortPath, AbsPathOS0),
+    atom_string(AbsPathOS0, AbsPathOS).
+
+test(file_blob, Read == "% -*- mode: Prolog; coding: utf-8 -*-\n\n") :-
+    my_file_open(File, my_program_home('test_cpp.pl'), 'r', [search,absolute,ospath]),
+    print(File), nl,
+    absolute_file_name(my_program_home('test_cpp.pl'), Abs, [access(read)]),
+    expected_file_name_my_program_home('test_cpp.pl', AbsPathOS_atom),
+    assertion(AbsPathOS_atom == Abs),
+    my_file_filename_atom(File, Filename),
+    assertion(Filename == Abs),
+    my_file_read(File, 39, Read),
+    my_file_close(File).
+% TODO: the following uses uninstantiated F, so explicit catch/3 is done
+% test(file_blob, error(existence_error(my_file_blob_open,F))) :-
+%     expected_file_name_my_program_home('non-existent-file', F0),
+%     atom_string(F0, F),
+%     my_file_open(_File, my_program_home('non-existent-file'), 'r', [search,absolute,ospath]).
+test(file_blob) :-
+    expected_file_name_my_program_home_string('non-existent-file', F),
+    catch(my_file_open(_File, my_program_home('non-existent-file'), 'r', [search,absolute,ospath]),
+          E,
+          assertion(E =@= error(existence_error(my_file_blob_open,F),_))).
+test(file_blob, error(existence_error(source_sink,my_program_home('non-existent-file')))) :-
+    my_file_open(_File, my_program_home('non-existent-file'), 'r', [search,absolute,ospath,read]).
+test(file_blob, error(existence_error(source_sink,my_program_home('non-existent-file')))) :-
+    absolute_file_name(my_program_home('non-existent-file'), _Abs, [access(read)]).
+
+test(option_flags, error(domain_error('MyFileBlob-options',foo))) :-
+    my_file_open(_File, my_program_home('test_cpp.pl'), 'r', [foo,search,absolute,ospath,read]).
+test(option_flags, error(type_error('atom or string',access(read)))) :-
+    my_file_open(_File, my_program_home('test_cpp.pl'), 'r', [search,absolute,ospath,access(read)]).
+
 test(nchars_flags, F-S == 0x43f-"xinteger,all") :-
     nchars_flags([xinteger,all,atomic,number], F),
     nchars_flags_string(F, S).
@@ -1058,6 +1107,9 @@ test_setup_call_cleanup(X) :-
         throw(error)).
 
 % Experimental API tests
+:- if((current_prolog_flag(version,V),V>=90308)).
+% Scoped terms depend on a working PL_free_term_ref()
+% implementation.  9.2.6 and up only provide a dummy.
 test(plterm_scoped, R == []) :-
     unify_atom_list([], R).
 test(plterm_scoped, R == [a, foo]) :-
@@ -1074,6 +1126,7 @@ test(plterm_scoped, error(type_error(list,foo))) :-
 
 test(plterm_scoped, [blocked('crashes in PL_free_term_ref')]) :-
     term_release.
+:- endif.
 
 test(record_ext, P == foo(bar,1,"a\0bc",'xy\0')) :-
     record_ext(foo(bar,1,"a\0bc", 'xy\0'), Str),
