@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        jan@swi-prolog.org
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2023, SWI-Prolog Solutions b.v.
+    Copyright (c)  2023-2024, SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -32,14 +32,6 @@
     POSSIBILITY OF SUCH DAMAGE.
 */
 
-static PyObject *PyExcProlog_store = NULL;
-static PyObject *
-PyExcProlog(void)
-{ if ( !PyExcProlog_store )
-    PyExcProlog_store = PyErr_NewException("janus.PrologError", NULL, NULL);
-  return PyExcProlog_store;
-}
-
 static PyObject *
 mod_janus(void)
 { static PyObject *janus = NULL;
@@ -70,7 +62,7 @@ Py_SetPrologErrorFromObject(PyObject *obj)
     PyTuple_SetItem(argv, 0, obj);
     PyObject *ex = PyObject_CallObject(constructor, argv);
     if ( ex )
-    { PyErr_SetObject(PyExcProlog(), ex);
+    { PyErr_SetObject(constructor, ex);
       Py_DECREF(ex);
     }
   }
@@ -417,7 +409,7 @@ swipl_open_query(PyObject *self, PyObject *args)
 
       out = PyList_New(STATE_LIST_LENGTH);
       tuple_set_int(0, out, fid);
-      tuple_set_int(1, out, (int64_t)qid);
+      tuple_set_int(1, out, (int64_t)(uintptr_t)qid);
       tuple_set_int(2, out, av);
       tuple_set_int(3, out, keep_bindings(args));
 
@@ -463,7 +455,7 @@ query_parms(PyObject *args, PyObject **tpp, fid_t *fid, qid_t *qid, term_t *av,
        !Py_GetInt64Arg(3, tp, &tav[3]) )
     return FALSE;
   *fid  = (fid_t)tav[0];
-  *qid  = (qid_t)tav[1];
+  *qid  = (qid_t)(uintptr_t)tav[1];
   *av   = (term_t)tav[2];
   *keep = (int)tav[3];
 
@@ -496,7 +488,14 @@ swipl_next_solution(PyObject *self, PyObject *args)
       done = TRUE;
       /*FALLTHROUGH*/
     case PL_S_TRUE:
-      py_from_prolog(av+2, &out);
+      if ( !py_from_prolog(av+2, &out) )
+      { term_t ex = PL_exception(0);
+
+	assert(ex);
+	ex = PL_copy_term_ref(ex);
+	PL_clear_exception();
+	Py_SetPrologError(ex);
+      }
       break;
     case PL_S_FALSE:
       out = PyBool_FromLong(0);
