@@ -2,7 +2,7 @@
 
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@cs.vu.nl
-    WWW:           http://www.swi-prolog.org/projects/xpce/
+    WWW:           http://www.swi-prolog.org/packages/xpce/
     Copyright (c)  1985-2018, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
@@ -51,6 +51,7 @@
 :- emacs_begin_mode(language, fundamental,
                     "Edit (programming) languages",
         [ indent_line                   = key('TAB'),
+          newline_and_indent		= key('\\ej'),
           backward_delete_char_untabify = key(backspace),
           align_close_bracket           = key(']') + key('}') + key(')'),
           insert_file_header            = key('\\C-c\\C-f'),
@@ -99,7 +100,42 @@ line_comment(E, CS:name) :<-
     get(Syntax, comment_end, CSlen, CE),
     send(CE, equal, '\n').
 
-insert_line_comment(E) :->
+line_comment_column_in_context(E) :->
+    get(E, line_comment_column_in_context, Col),
+    send(E, report, inform, 'Column %d', Col).
+
+line_comment_start(TB, Re, Caret, To, Start) :-
+    send(Re, search, TB, Caret, To),
+    get(Re, register_start, 1, Start).
+
+line_comment_column_in_context(E, Col:int) :<-
+    "Find the column for line comment nearby"::
+    member(CSlen, [1, 2]),
+    get(E?syntax, comment_start, CSlen, CS),
+    !,
+    get(E, caret, Caret),
+    get(E, text_buffer, TB),
+    get(TB, size, TBLen),
+    get(regex(''), quote, CS, string(QCS)),
+    format(string(ReS), '^\\s*\\S[^\n]*(~w)', [QCS]),
+    new(Re, regex(ReS)),
+    Start is max(Caret-10000, 0),
+    End is min(Caret+10000, TBLen),
+    ignore(line_comment_start(TB, Re, Caret, Start, M1)),
+    ignore(line_comment_start(TB, Re, Caret, End,   M2)),
+    (   nonvar(M1), nonvar(M2)
+    ->  (   abs(M1-Caret) < abs(M2-Caret)
+        ->  SCM = M1
+        ;   SCM = M2
+        )
+    ;   nonvar(M1)
+    ->  SCM = M1
+    ;   nonvar(M2)
+    ->  SCM = M2
+    ),
+    get(E, column, SCM, Col).
+
+insert_line_comment(E, Arg:[int]) :->
     "Insert (line) comment"::
     member(CSlen, [1, 2]),
     get(E?syntax, comment_start, CSlen, CS),
@@ -108,13 +144,21 @@ insert_line_comment(E) :->
     get(E, text_buffer, TB),
     get(TB, scan, Caret, line, 0, start, SOL),
     get(TB, scan, Caret, line, 0, end,   EOL),
+    (   integer(Arg), Arg > 4
+    ->  CC = Arg
+    ;   Arg == @default,
+        get(E, line_comment_column_in_context, CC),
+        CC > 20, CC < 100
+    ->  true
+    ;   get(E, comment_column, CC)
+    ),
     (   get(regex(?(regex(''), quote, CS)), search, TB, SOL, EOL, Start)
     ->  send(E, caret, Start),
-        send(E, align, E?comment_column),
+        send(E, align, CC),
         send(E, forward_char, CSlen + 1)
     ;   send(E, end_of_line),
         send(E, just_one_space),
-        send(E, align, E?comment_column),
+        send(E, align, CC),
         get(E?syntax, comment_end, CSlen, CE),
         (   send(CE, equal, '\n')
         ->  send(E, format, '%s ', CS)
@@ -458,7 +502,7 @@ indent_close_bracket_line(E, Brackets:[name], Base:[int]) :->
     ->  OpenPos >= Base
     ;   true
     ),
-    (   get(E, argument_indent, OpenPos, Col)
+    (   get(E, argument_indent, OpenPos, OpenPos, Col)
     ->  true
     ;   get(E, column, OpenPos, Col)
     ),
@@ -499,7 +543,7 @@ indent_expression_line(E, Brackets:[name], Base:[int]) :->
         ),
         (   send(E, looking_at, '[,|]')     % line starts with , or |
         ->  get(E, column, OpenPos, Col)
-        ;   get(E, argument_indent, OpenPos, StartCol)
+        ;   get(E, argument_indent, OpenPos, Base, StartCol)
         ->  get(E, parameter_indentation, PI),
             Col is StartCol+PI
         ;   get(TB, scan, OpenPos, line, 0, end, EOL),
@@ -510,13 +554,17 @@ indent_expression_line(E, Brackets:[name], Base:[int]) :->
     !.
 
 
-argument_indent(E, OpenPos:int, StartCol:int) :<-
+argument_indent(E, OpenPos:int, After:[int], StartCol:int) :<-
     "Get column for indented arguments from OpenPos"::
     get(E, text_buffer, TB),
     get(TB, scan, OpenPos, line, 0, end, EOL),
     get(TB, skip_comment, OpenPos+1, EOL, P1),
     P1 == EOL,
     get(E, scan, OpenPos, term, -1, start, SOT),
+    (   After \== @default
+    ->  SOT > After
+    ;   true
+    ),
     get(E, column, SOT, StartCol).
 
 
