@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org/packages/xpce/
-    Copyright (c)  2011-2019, University of Amsterdam
+    Copyright (c)  2011-2024, University of Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -115,7 +116,7 @@
 				  (type *) alloca((size)*sizeof(type))
 #endif
 
-typedef void (*OnExitFunction)(void);	/* HOST_ONEXIT function */
+typedef int (*OnExitFunction)(int rc);	/* HOST_ONEXIT function */
 
 		 /*******************************
 		 *    PROLOG ELEMENTARY TYPES	*
@@ -686,6 +687,10 @@ install_pl2xpce(void)
   PL_register_foreign("pce_postscript_stream", 1,
 		      pl_pce_postscript_stream, 0);
 
+#ifndef __WINDOWS__
+  PL_license("lgplv2+", "xpce (drag&drop library by Paul Sheer)");
+#endif
+
   install_pcecall();
 }
 
@@ -703,7 +708,7 @@ uninstall_pl2xpce(void)
   indirect_rlc_update_hook(old_update_hook);
 #endif
   if ( exitpce_hook )
-    (*exitpce_hook)();
+    (*exitpce_hook)(0);
 }
 
 #endif /*O_SHAREDLIBRARY*/
@@ -3031,7 +3036,13 @@ pl_Cgetline(char *buf, int size)
 		 *******************************/
 
 typedef void (*sighandler_t)(int);
-typedef int  (*halthandler_t)(int, void *);
+
+static int
+swi_halt_hook(int rc, void *closure)
+{ OnExitFunction f = closure;
+
+  return (*f)(rc);
+}
 
 static int
 PrologAction(int action, va_list args)
@@ -3069,7 +3080,7 @@ PrologAction(int action, va_list args)
     case HOST_ATEXIT:
     { OnExitFunction f = va_arg(args, OnExitFunction);
 
-      PL_on_halt((halthandler_t)f, NULL);
+      PL_on_halt(swi_halt_hook, f);
 
       return PCE_SUCCEED;
     }
