@@ -3,8 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2022, University of Amsterdam
-                              VU University Amsterdam
+    Copyright (c)  2011-2024, University of Amsterdam
+			      VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
@@ -39,7 +39,6 @@ Formatted output (Prolog predicates format/[1,2,3]).   One  day,  the  C
 source should also use format() to produce error messages, etc.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#include "pl-fmt.h"
 #include "pl-ctype.h"
 #include "pl-utf8.h"
 #include "../pl-arith.h"
@@ -50,6 +49,13 @@ source should also use format() to produce error messages, etc.
 #include "../pl-pro.h"
 #include <ctype.h>
 #include <stdio.h>
+#include <math.h>
+#include <fenv.h>
+#ifdef __WINDOWS__
+#include "../pl-nt.h"
+#endif
+
+typedef foreign_t (*Func1)(term_t a1);
 
 static char *	formatInteger(PL_locale *locale, int div, int radix,
 			     bool smll, Number n, Buffer out);
@@ -82,7 +88,7 @@ typedef struct
 			}
 #define FMT_ERROR(fmt)	return PL_error(NULL, 0, NULL, ERR_FORMAT, fmt)
 #define FMT_ARG(c, a)	return PL_error(NULL, 0, NULL, ERR_FORMAT_ARG, c, a)
-#define FMT_EXEPTION()	return FALSE
+#define FMT_EXEPTION()	return false
 
 
 static PL_locale prolog_locale =
@@ -120,20 +126,18 @@ outchr(format_state *state, int chr)
       for(s=buf; s<e; s++)
 	addBuffer((Buffer)&state->buffer, *s, char);
     } else
-    { char c = chr;
-
-      addBuffer((Buffer)&state->buffer, c, char);
+    { addBuffer((Buffer)&state->buffer, (char)chr, char);
     }
 
     state->buffered++;
   } else
   { if ( Sputcode(chr, state->out) < 0 )
-      return FALSE;
+      return false;
   }
 
   state->column = update_column(state->column, chr);
 
-  return TRUE;
+  return true;
 }
 
 
@@ -153,14 +157,14 @@ outstring(format_state *state, const char *s, size_t len)
   } else
   { for(q=s; q < e; q++)
     { if ( Sputcode(*q&0xff, state->out) < 0 )
-	return FALSE;
+	return false;
     }
   }
 
   for(q=s; q < e; q++)
     state->column = update_column(state->column, *q&0xff);
 
-  return TRUE;
+  return true;
 }
 
 
@@ -173,10 +177,10 @@ oututf8(format_state *state, const char *s, size_t len)
 
     PL_utf8_code_point(&s, e, &chr);
     if ( !outchr(state, chr) )
-      return FALSE;
+      return false;
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -200,14 +204,14 @@ outtext(format_state *state, PL_chars_t *txt)
 
 	s = get_wchar(s, &c);
 	if ( !outchr(state, c) )
-	  return FALSE;
+	  return false;
       }
 
-      return TRUE;
+      return true;
     }
     default:
     { assert(0);
-      return FALSE;
+      return false;
     }
   }
 }
@@ -226,62 +230,70 @@ static WUNUSED int emit_rubber(format_state *state);
 		*       PROLOG CONNECTION	*
 		********************************/
 
-word
-pl_format_predicate(term_t chr, term_t descr)
-{ int c;
+static
+PRED_IMPL("format_predicate", 2, format_predicate, META)
+{ PRED_LD
+  int c;
   predicate_t proc = NULL;
   size_t arity;
 
-  if ( !PL_get_char_ex(chr, &c, FALSE) )
-    fail;
+  if ( !PL_get_char_ex(A1, &c, false) )
+    return false;
+  if ( !get_procedure(A2, &proc, 0, GP_CREATE) )
+    return false;
 
-  if ( !get_procedure(descr, &proc, 0, GP_CREATE) )
-    fail;
   PL_predicate_info(proc, NULL, &arity, NULL);
   if ( arity == 0 )
     return PL_error(NULL, 0, "arity must be > 0", ERR_DOMAIN,
 		    PL_new_atom("format_predicate"),
-		    descr);
+		    A2);
 
   if ( !format_predicates )
-    format_predicates = newHTable(8);
+    format_predicates = newHTableWP(8);
 
-  updateHTable(format_predicates, (void *)(intptr_t)c, proc);
+  updateHTableWP(format_predicates, c, proc);
 
-  succeed;
+  return true;
 }
 
 
-word
-pl_current_format_predicate(term_t chr, term_t descr, control_t h)
-{ GET_LD
-  intptr_t name;
-  predicate_t pred;
+static
+PRED_IMPL("current_format_predicate", 2, current_format_predicate, NDET|META)
+{ PRED_LD
   TableEnum e;
   fid_t fid;
 
-  switch( ForeignControl(h) )
+  term_t chr = A1;
+  term_t descr = A2;
+
+  switch( CTX_CNTRL )
   { case FRG_FIRST_CALL:
       if ( !format_predicates )
 	fail;
-      e = newTableEnum(format_predicates);
+      e = newTableEnumWP(format_predicates);
       break;
     case FRG_REDO:
-      e = ForeignContextPtr(h);
+      e = CTX_PTR;
       break;
     case FRG_CUTTED:
-      e = ForeignContextPtr(h);
+      e = CTX_PTR;
       freeTableEnum(e);
     default:
-      succeed;
+      return true;
   }
 
   if ( !(fid = PL_open_foreign_frame()) )
   { freeTableEnum(e);
-    return FALSE;
+    return false;
   }
-  while( advanceTableEnum(e, (void**)&name, (void**)&pred) )
-  { if ( PL_unify_integer(chr, name) &&
+
+  table_key_t tk;
+  table_value_t tv;
+  while( advanceTableEnum(e, &tk, &tv) )
+  { int c = (int)tv;
+    predicate_t pred = val2ptr(tv);
+
+    if ( PL_unify_integer(chr, c) &&
 	 PL_unify_predicate(descr, pred, 0) )
     { PL_close_foreign_frame(fid);
       ForeignRedoPtr(e);
@@ -292,11 +304,11 @@ pl_current_format_predicate(term_t chr, term_t descr, control_t h)
 
   PL_close_foreign_frame(fid);
   freeTableEnum(e);
-  fail;
+  return false;
 }
 
 
-static word
+static foreign_t
 format_impl(IOSTREAM *out, term_t format, term_t Args, Module m)
 { GET_LD
   term_t argv;
@@ -308,7 +320,7 @@ format_impl(IOSTREAM *out, term_t format, term_t Args, Module m)
   if ( !PL_get_text(format, &fmt, CVT_ATOM|CVT_STRING|CVT_LIST|BUF_STACK) )
     return PL_error("format", 3, NULL, ERR_TYPE, ATOM_text, format);
 
-  if ( (argc = (int)lengthList(args, FALSE)) >= 0 )
+  if ( (argc = (int)lengthList(args, false)) >= 0 )
   { term_t head = PL_new_term_ref();
     int n = 0;
 
@@ -339,19 +351,19 @@ format_impl(IOSTREAM *out, term_t format, term_t Args, Module m)
   return rval;
 }
 
+#define format(out, fmt, args) LDFUNC(format, out, fmt, args)
 
-word
-pl_format3(term_t out, term_t format, term_t args)
-{ GET_LD
-  redir_context ctx;
-  word rc;
+static foreign_t
+format(DECL_LD term_t out, term_t format, term_t args)
+{ redir_context ctx;
+  foreign_t rc;
   Module m = NULL;
   term_t list = PL_new_term_ref();
 
   if ( !PL_strip_module(args, &m, list) )
-    return FALSE;
+    return false;
 
-  if ( (rc=setupOutputRedirect(out, &ctx, FALSE)) )
+  if ( (rc=setupOutputRedirect(out, &ctx, false)) )
   { if ( (rc = format_impl(ctx.stream, format, list, m)) )
       rc = closeOutputRedirect(&ctx);
     else
@@ -361,12 +373,19 @@ pl_format3(term_t out, term_t format, term_t args)
   return rc;
 }
 
+static
+PRED_IMPL("format", 2, format, META)
+{ PRED_LD
 
-word
-pl_format(term_t fmt, term_t args)
-{ return pl_format3(0, fmt, args);
+  return format(0, A1, A2);
 }
 
+static
+PRED_IMPL("format", 3, format, META)
+{ PRED_LD
+
+    return format(A1, A2, A3);
+}
 
 static inline int
 get_chr_from_text(const PL_chars_t *t, int index)
@@ -406,11 +425,11 @@ prepare_sub_format(sub_state *state, format_state *fstate, IOSTREAM *fd)
     state->bufsize    = sizeof(state->buf);
 
     tellString(&state->str, &state->bufsize, ENC_UTF8);
-    if ( true(fd, SIO_ISATTY) )
+    if ( ison(fd, SIO_ISATTY) )
       set(Scurout, SIO_ISATTY);
   }
 
-  return TRUE;
+  return true;
 }
 
 static int
@@ -423,9 +442,7 @@ end_sub_format(sub_state *state, int rc)
     if ( rc )
       state->fstate->column = lp;
   } else
-  { int lp = Scurout->position->linepos;
-
-    toldString();
+  { toldString();
     if ( rc )
     { int c0 = state->fstate->column;
       rc = oututf8(state->fstate, state->str, state->bufsize);
@@ -452,7 +469,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
   format_state state;			/* complete state */
   int tab_stop = 0;			/* padded tab stop */
   unsigned int here = 0;
-  int rc = TRUE;
+  int rc = true;
 
   state.out = fd;
   state.pending_rubber = 0;
@@ -470,7 +487,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
     switch(c)
     { case '~':
 	{ int arg = DEFAULT;		/* Numeric argument */
-	  int mod_colon = FALSE;	/* Used colon modifier */
+	  int mod_colon = false;	/* Used colon modifier */
 	  predicate_t proc;
 					/* Get the numeric argument */
 	  c = get_chr_from_text(fmt, ++here);
@@ -507,13 +524,13 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 	  }
 
 	  if ( c == ':' )
-	  { mod_colon = TRUE;
+	  { mod_colon = true;
 	    c = get_chr_from_text(fmt, ++here);
 	  }
 
 					/* Check for user defined format */
 	  if ( format_predicates &&
-	       (proc = lookupHTable(format_predicates, (void*)((intptr_t)c))) )
+	       (proc = lookupHTableWP(format_predicates, c)) )
 	  { size_t arity;
 	    term_t av;
 	    sub_state sstate;
@@ -552,7 +569,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		    FMT_ARG("a", argv);
 		  SHIFT;
 		  rc = outtext(&state, &txt);
-                  if ( !rc )
+		  if ( !rc )
 		    goto out;
 		  here++;
 		  break;
@@ -568,7 +585,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		    while(times-- > 0)
 		    { rc = outchr(&state, chr);
 		      if ( !rc )
-		        goto out;
+			goto out;
 		    }
 		  } else
 		    FMT_ARG("c", argv);
@@ -580,7 +597,8 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 	      case 'f':			/* float */
 	      case 'g':			/* shortest of 'f' and 'e' */
 	      case 'G':			/* shortest of 'f' and 'E' */
-		{ number n;
+		{ AR_CTX
+		  number n;
 		  union {
 		  tmp_buffer b;
 		    buffer b1;
@@ -588,12 +606,14 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  PL_locale *l;
 
 		  NEED_ARG;
+		  AR_BEGIN();
 		  if ( !valueExpression(argv, &n) )
 		  { char f[2];
 
-		    f[0] = c;
+		    f[0] = (char)c;
 		    f[1] = EOS;
-		    FMT_ARG(f, argv);
+		    AR_CLEANUP();
+		    FMT_ARG(f, argv); /* returns error */
 		  }
 		  SHIFT;
 
@@ -605,10 +625,11 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  initBuffer(&u.b);
 		  rc = formatFloat(l, c, arg, &n, &u.b1) != NULL;
 		  clearNumber(&n);
+		  AR_END();
 		  if ( rc )
 		    rc = oututf80(&state, baseBuffer(&u.b, char));
 		  discardBuffer(&u.b);
-                  if ( !rc )
+		  if ( !rc )
 		    goto out;
 		  here++;
 		  break;
@@ -618,16 +639,20 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 	      case 'r':			/* radix number */
 	      case 'R':			/* Radix number */
 	      case 'I':			/* Prolog 1_000_000 */
-		{ number i;
+		{ AR_CTX
+		  number i;
 		  tmp_buffer b;
+		  char *si;
 
 		  NEED_ARG;
+		  AR_BEGIN();
 		  if ( !valueExpression(argv, &i) ||
 		       !toIntegerNumber(&i, 0) )
 		  { char f[2];
 
-		    f[0] = c;
+		    f[0] = (char)c;
 		    f[1] = EOS;
+		    AR_CLEANUP();
 		    FMT_ARG(f, argv);
 		  }
 		  SHIFT;
@@ -650,19 +675,17 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 
 		    if ( arg == DEFAULT )
 		      arg = 0;
-		    if ( !formatInteger(l, arg, 10, TRUE, &i, (Buffer)&b) )
-		      FMT_EXEPTION();
+		    si = formatInteger(l, arg, 10, true, &i, (Buffer)&b);
 		  } else if ( c == 'I' )
 		  { PL_locale ltmp;
 		    char grouping[2];
 
-		    grouping[0] = (arg == DEFAULT ? 3 : arg);
+		    grouping[0] = (char)(arg == DEFAULT ? 3 : arg);
 		    grouping[1] = '\0';
 		    ltmp.thousands_sep = L"_";
 		    ltmp.grouping = grouping;
 
-		    if ( !formatInteger(&ltmp, 0, 10, TRUE, &i, (Buffer)&b) )
-		      FMT_EXEPTION();
+		    si = formatInteger(&ltmp, 0, 10, true, &i, (Buffer)&b);
 		  } else			/* r,R */
 		  { if ( arg == DEFAULT )
 		      arg = 8;
@@ -670,14 +693,17 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		    { term_t r = PL_new_term_ref();
 
 		      PL_put_integer(r, arg);
-		      return PL_error(NULL, 0, NULL, ERR_DOMAIN,
-				      ATOM_radix, r);
-		    }
-		    if ( !formatInteger(NULL, 0, arg, c == 'r', &i, (Buffer)&b) )
-		      FMT_EXEPTION();
+		      PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_radix, r);
+		      si = NULL;
+		    } else
+		      si = formatInteger(NULL, 0, arg, c == 'r', &i, (Buffer)&b);
 		  }
 		  clearNumber(&i);
-		  rc = oututf80(&state, baseBuffer(&b, char));
+		  AR_END();
+		  if ( si )
+		    rc = oututf80(&state, si);
+		  else
+		    rc = false;
 		  discardBuffer(&b);
 		  if ( !rc )
 		    goto out;
@@ -710,18 +736,18 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  here++;
 		  break;
 		}
-		{ Func f;
+		{ Func1 f;
 		  sub_state sstate;
 
 	      case 'k':			/* write_canonical */
 		  f = pl_write_canonical;
-	          goto pl_common;
+		  goto pl_common;
 	      case 'p':			/* print */
 		  f = pl_print;
-	          goto pl_common;
+		  goto pl_common;
 	      case 'q':			/* writeq */
 		  f = pl_writeq;
-	          goto pl_common;
+		  goto pl_common;
 	      case 'w':			/* write */
 		  f = pl_write;
 		  pl_common:
@@ -761,7 +787,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		 break;
 	       }
 	      case '@':
-	        { term_t ex = 0;
+		{ term_t ex = 0;
 		  sub_state sstate;
 		  fid_t fid;
 
@@ -787,7 +813,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  SHIFT;
 		  here++;
 		  break;
-	        }
+		}
 	      case '~':			/* ~ */
 		{ rc = outchr(&state, '~');
 		  if ( !rc )
@@ -802,10 +828,10 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  if ( c == 'N' && state.column == 0 )
 		    arg--;
 		  while( arg-- > 0 )
-                  { rc = outchr(&state, '\n');
+		  { rc = outchr(&state, '\n');
 		    if ( !rc )
 		      goto out;
-                  }
+		  }
 		  here++;
 		  break;
 		}
@@ -853,7 +879,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 
 		  if ( nl_and_reindent )
 		  { if ( Sputcode('\n', state.out) < 0 )
-		    { rc = FALSE;
+		    { rc = false;
 		      goto out;
 		    }
 		    state.column = update_column(state.column, '\n');
@@ -945,7 +971,7 @@ emit_rubber(format_state *state)
 
       for(n=0; n<r->size; n++)
       { if ( Sputcode(r->pad, state->out) < 0 )
-	  return FALSE;
+	  return false;
       }
       r++;
       rn--;
@@ -954,7 +980,7 @@ emit_rubber(format_state *state)
     if ( s < e )
     { PL_utf8_code_point(&s, e, &chr);
       if ( Sputcode(chr, state->out) < 0 )
-	return FALSE;
+	return false;
     } else
       break;
   }
@@ -964,7 +990,7 @@ emit_rubber(format_state *state)
   state->buffered = 0;
   state->pending_rubber = 0;
 
-  return TRUE;
+  return true;
 }
 
 
@@ -985,7 +1011,7 @@ lappend(const wchar_t *l, int def, Buffer out)
     { int c = *e;
 
       if ( c < 128 )
-      { addBuffer(out, c, char);
+      { addBuffer(out, (char)c, char);
       } else
       { char buf[6];
 	char *e8, *s;
@@ -997,7 +1023,7 @@ lappend(const wchar_t *l, int def, Buffer out)
       }
     }
   } else
-  { addBuffer(out, def, char);
+  { addBuffer(out, (char)def, char);
   }
 }
 
@@ -1006,7 +1032,7 @@ revert_string(char *s, size_t len)
 { char *e = &s[len-1];
 
   for(; e>s; s++,e--)
-  { int c = *e;
+  { char c = *e;
 
     *e = *s;
     *s = c;
@@ -1033,8 +1059,8 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
       if ( n == 0 && div == 0 )
       { addBuffer(out, '0', char);
       } else
-      { int before = FALSE;			/* before decimal point */
-	int negative = FALSE;
+      { int before = false;			/* before decimal point */
+	int negative = false;
 	int gsize = 0;
 	int dweight;
 
@@ -1044,7 +1070,7 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
 	{ if ( div-- == 0 && !before )
 	  { if ( !isEmptyBuffer(out) )
 	      lappend(locale->decimal_point, '.', out);
-	    before = TRUE;
+	    before = true;
 	    if ( grouping )
 	      gsize = grouping[0];
 	  }
@@ -1076,25 +1102,24 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
 
       return baseBuffer(out, char);
     }
-#ifdef O_GMP
+#ifdef O_BIGNUM
     case V_MPZ:
     { GET_LD
-      size_t len = mpz_sizeinbase(i->value.mpz, radix);
+	size_t len = (size_t)((double)mpz_sizeinbase(i->value.mpz, 2) *
+			      log(radix)/log(2) * 1.2);
       char tmp[256];
       char *buf;
-      int rc = TRUE;
+      int rc;
 
       if ( len+2 > sizeof(tmp) )
-	buf = PL_malloc(len+2);
+	buf = tmp_malloc(len+2);
       else
 	buf = tmp;
 
-      EXCEPTION_GUARDED({ LD->gmp.persistent++;
-			  mpz_get_str(buf, radix, i->value.mpz);
-			  LD->gmp.persistent--;
+      EXCEPTION_GUARDED({ mpz_get_str(buf, radix, i->value.mpz);
+			  rc = true;
 			},
-			{ LD->gmp.persistent--;
-			  rc = PL_rethrow();
+			{ rc = false;
 			});
       if ( !rc )
 	return NULL;
@@ -1103,11 +1128,11 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
       { char *s;
 
 	for(s=buf; *s; s++)
-	  *s = toupper(*s);
+	  *s = (char)toupper(*s);
       }
 
       if ( grouping || div > 0 )
-      { int before = FALSE;			/* before decimal point */
+      { int before = false;			/* before decimal point */
 	int gsize = 0;
 	char *e = buf+strlen(buf)-1;
 
@@ -1115,7 +1140,7 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
 	{ if ( div-- == 0 && !before )
 	  { if ( !isEmptyBuffer(out) )
 	      lappend(locale->decimal_point, '.', out);
-	    before = TRUE;
+	    before = true;
 	    if ( grouping )
 	      gsize = grouping[0];
 	  }
@@ -1139,12 +1164,12 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
       }
 
       if ( buf != tmp )
-	PL_free(buf);
+	tmp_free(buf);
 
       addBuffer(out, EOS, char);
       return baseBuffer(out, char);
     }
-#endif /*O_GMP*/
+#endif /*O_BIGNUM*/
     default:
       assert(0);
       return NULL;
@@ -1194,11 +1219,11 @@ static int
 same_decimal_point(PL_locale *l1, PL_locale *l2)
 { if ( l1->decimal_point && l2->decimal_point &&
        wcscmp(l1->decimal_point, l2->decimal_point) == 0 )
-    return TRUE;
+    return true;
   if ( !l1->decimal_point && !l2->decimal_point )
-    return TRUE;
+    return true;
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1206,7 +1231,7 @@ static int
 utf8_dp(PL_locale *l, char *s, int *len)
 { if ( l->decimal_point )
   { if ( !ths_to_utf8(s, l->decimal_point, 20) )
-      return FALSE;
+      return false;
     *len = (int)strlen(s);
   } else
   { *s++ = '.';
@@ -1214,7 +1239,7 @@ utf8_dp(PL_locale *l, char *s, int *len)
     *len = 1;
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1228,7 +1253,7 @@ static int
 localizeDecimalPoint(PL_locale *locale, Buffer b)
 { if ( locale == GD->locale.default_locale ||
        same_decimal_point(GD->locale.default_locale, locale) )
-    return TRUE;
+    return true;
 
   if ( locale->decimal_point && locale->decimal_point[0] )
   { char *s = baseBuffer(b, char);
@@ -1238,7 +1263,7 @@ localizeDecimalPoint(PL_locale *locale, Buffer b)
 
     if ( !utf8_dp(locale, dp, &dplen) ||
 	 !utf8_dp(GD->locale.default_locale, ddp, &ddplen) )
-      return FALSE;
+      return false;
 
     if ( *s == '-' )
       s++;
@@ -1260,7 +1285,7 @@ localizeDecimalPoint(PL_locale *locale, Buffer b)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1287,7 +1312,7 @@ groupDigits(PL_locale *locale, Buffer b)
       int thslen;
 
       if ( !ths_to_utf8(ths, locale->thousands_sep, sizeof(ths)) )
-	return FALSE;
+	return false;
       thslen = (int)strlen(ths);
 
       if ( !growBuffer(b, thslen*groups) )
@@ -1312,7 +1337,7 @@ groupDigits(PL_locale *locale, Buffer b)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -1354,54 +1379,54 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
     case V_MPZ:
     { switch(how)
       { case 'f':
-        { mpz_init(t1);
-          mpz_init(t2);
-          mpz_ui_pow_ui(t1, 10, arg);
-          mpz_mul(t1, f->value.mpz, t1);
-          neg = (mpz_cmp_ui(t1, 0) < 0) ? 1 : 0;
-          mpz_abs(t1, t1);
-          goto print_mpz_f;
-        }
-        case 'e':
+	{ mpz_init(t1);
+	  mpz_init(t2);
+	  mpz_ui_pow_ui(t1, 10, arg);
+	  mpz_mul(t1, f->value.mpz, t1);
+	  neg = mpz_sgn(t1) < 0;
+	  mpz_abs(t1, t1);
+	  goto print_mpz_f;
+	}
+	case 'e':
 	case 'E':
-        { mpz_init(t1);
-          mpz_init(t2);
+	{ mpz_init(t1);
+	  mpz_init(t2);
 
-          mpz_set(t1, f->value.mpz);
-          neg = (mpz_cmp_ui(t1, 0) < 0) ? 1 : 0;
-          mpz_abs(t1, t1);
+	  mpz_set(t1, f->value.mpz);
+	  neg = mpz_sgn(t1) < 0;
+	  mpz_abs(t1, t1);
 
-          if (mpz_cmp_ui(t1, 0) == 0)
-          { exp = 0;
-          } else
-          { exp = mpz_sizeinbase(t1, 10)-1;  /* guess exponent */
-            mpz_ui_pow_ui(t2, 10, exp);
-            if (mpz_cmp(t2, t1) > 0) exp--;  /* correct exponent */
+	  if ( mpz_sgn(t1) == 0 )
+	  { exp = 0;
+	  } else
+	  { exp = mpz_sizeinbase(t1, 10)-1;  /* guess exponent */
+	    mpz_ui_pow_ui(t2, 10, exp);
+	    if (mpz_cmp(t2, t1) > 0) exp--;  /* correct exponent */
 
-            if (exp > arg)
-            { mpz_ui_pow_ui(t2, 10, (exp-arg-1));
-              mpz_tdiv_q(t1, t1, t2);
-              mpz_add_ui(t1, t1, 5);
-              mpz_tdiv_q_ui(t1, t1, 10);
-            } else
-            { mpz_ui_pow_ui(t2, 10, arg-exp);
-              mpz_mul(t1, t1, t2);
-            }
-          }
-          mpz_set_ui(t2, exp);
-          goto print_mpz_e;
-        }
-        case 'g':
+	    if (exp > arg)
+	    { mpz_ui_pow_ui(t2, 10, (exp-arg-1));
+	      mpz_tdiv_q(t1, t1, t2);
+	      mpz_add_ui(t1, t1, 5);
+	      mpz_tdiv_q_ui(t1, t1, 10);
+	    } else
+	    { mpz_ui_pow_ui(t2, 10, arg-exp);
+	      mpz_mul(t1, t1, t2);
+	    }
+	  }
+	  mpz_set_ui(t2, exp);
+	  goto print_mpz_e;
+	}
+	case 'g':
 	case 'G':
-        { mpf_init2(mpf, arg*4);
-          mpf_set_z(mpf, f->value.mpz);
-          goto print_mpf;
-        }
+	{ mpf_init2(mpf, arg*4);
+	  mpf_set_z(mpf, f->value.mpz);
+	  goto print_mpf;
+	}
       }
     }
     case V_MPQ:
     { char tmp[12];
-      int size;
+      size_t size;
       int written;
       int fbits;
       int digits;
@@ -1409,210 +1434,269 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
 
       switch(how)
       { case 'f':
-        { mpz_init(t1);
-          mpz_init(t2);
-          mpz_ui_pow_ui(t1, 10, arg);
-          mpz_mul(t1, mpq_numref(f->value.mpq), t1);
-          mpz_tdiv_q_ui(t2, mpq_denref(f->value.mpq), 2);
-          if (mpq_cmp_ui(f->value.mpq, 0, 1) < 0)
-          { mpz_sub(t1, t1, t2);
-            neg=1;
-          } else
-          { mpz_add(t1, t1, t2);
-            neg=0;
-          }
-          mpz_tdiv_q(t1, t1, mpq_denref(f->value.mpq));
-          mpz_abs(t1, t1);
+	{ mpz_init(t1);
+	  mpz_init(t2);
+	  mpz_ui_pow_ui(t1, 10, arg);
+	  mpz_mul(t1, mpq_numref(f->value.mpq), t1);
+	  mpz_tdiv_q_ui(t2, mpq_denref(f->value.mpq), 2);
+	  if (mpq_cmp_ui(f->value.mpq, 0, 1) < 0)
+	  { mpz_sub(t1, t1, t2);
+	    neg=1;
+	  } else
+	  { mpz_add(t1, t1, t2);
+	    neg=0;
+	  }
+	  mpz_tdiv_q(t1, t1, mpq_denref(f->value.mpq));
+	  mpz_abs(t1, t1);
 
-        print_mpz_f:
+	print_mpz_f:
 
-          if (mpz_cmp_ui(t1, 0) != 0)
-          { size = mpz_sizeinbase(t1, 10) + 1; /* reserve for <null> */
-            if ( !growBuffer(out, size) )
-            { PL_no_memory();
-              return NULL;
-            }
-            digits = written = gmp_snprintf(baseBuffer(out, char),
-					    size, "%Zd", t1);
-          } else
+	  if ( mpz_sgn(t1) != 0 )
+	  { size =(size_t)((double)mpz_sizeinbase(t1, 2) * log(10)/log(2) * 1.2 + 1);
+	    if ( !growBuffer(out, size) )
+	    { PL_no_memory();
+	      return NULL;
+	    }
+	    mpz_get_str(baseBuffer(out, char), 10, t1);
+	    digits = written = strlen(baseBuffer(out, char));
+	  } else
 	  { written = digits = 0;
 	  }
-          if ( digits <= arg )
+	  if ( digits <= arg )
 	    padding = (arg-digits+1);
 	  else
 	    padding = 0;
 
-          size = digits;
-          if (neg) size++;               /* leading - */
-          if (arg) size++;               /* decimal point */
-          if (padding) size += padding;  /* leading '0's */
-          size++;                        /* NULL terminator */
+	  size = digits;
+	  if (neg) size++;               /* leading - */
+	  if (arg) size++;               /* decimal point */
+	  if (padding) size += padding;  /* leading '0's */
+	  size++;                        /* NULL terminator */
 
-          if ( !growBuffer(out, size) )
-          { PL_no_memory();
-            return NULL;
-          }
+	  if ( !growBuffer(out, size) )
+	  { PL_no_memory();
+	    return NULL;
+	  }
 
-          if (!digits)
-          { memset(out->base, '\0', 1);
-          }
+	  if (!digits)
+	  { memset(out->base, '\0', 1);
+	  }
 
-          if (padding)
-          { memmove(out->base+padding, out->base, written+1);
-            memset(out->base, '0', padding);
-            written += padding;
-          }
+	  if (padding)
+	  { memmove(out->base+padding, out->base, written+1);
+	    memset(out->base, '0', padding);
+	    written += padding;
+	  }
 
-          if (arg)
-          { memmove(out->base+written-(arg-1), out->base+written-arg, arg+1);
-            if ( locale->decimal_point && locale->decimal_point[0] )
-              *(out->base+written-arg) = locale->decimal_point[0];
-            else
-              *(out->base+written-arg) = '.';
-            written++;
-          }
+	  if (arg)
+	  { memmove(out->base+written-(arg-1), out->base+written-arg, arg+1);
+	    if ( locale->decimal_point && locale->decimal_point[0] )
+	      *(out->base+written-arg) = (char)locale->decimal_point[0];
+	    else
+	      *(out->base+written-arg) = '.';
+	    written++;
+	  }
 
-          if (neg)
-          { memmove(out->base+1, out->base, written+1);
-            memset(out->base, '-', 1);
-            written++;
-          }
+	  if (neg)
+	  { memmove(out->base+1, out->base, written+1);
+	    memset(out->base, '-', 1);
+	    written++;
+	  }
 
-          out->top = out->base + written;
-          mpz_clear(t1);
-          mpz_clear(t2);
-          break;
-        }
-        case 'e':
+	  out->top = out->base + written;
+	  mpz_clear(t1);
+	  mpz_clear(t2);
+	  break;
+	}
+	case 'e':
 	case 'E':
-        { mpz_init(t1);
-          mpz_init(t2);
+	{ mpz_init(t1);
+	  mpz_init(t2);
 
-          if (mpz_cmpabs(mpq_numref(f->value.mpq), mpq_denref(f->value.mpq)) >= 0)
-          { mpz_tdiv_q(t1, mpq_numref(f->value.mpq), mpq_denref(f->value.mpq));
-            exp = mpz_sizeinbase(t1, 10)-1;     /* guess exponent */
-            mpz_ui_pow_ui(t2, 10, exp);
-            if (mpz_cmpabs(t2, t1) > 0) exp--;  /* correct exponent */
-          } else
-          { mpq_t tq1, tq2;
-            mpq_init(tq1);
-            mpq_init(tq2);
-            mpz_set(t1, mpq_numref(f->value.mpq));
-            mpz_abs(t1, t1);
-            mpz_tdiv_q(t1, mpq_denref(f->value.mpq), t1);
-            exp = 0-mpz_sizeinbase(t1, 10);     /* guess exponent */
-            mpz_ui_pow_ui(t2, 10, abs(exp)-1);
-            mpq_set_z(tq1, t1);
-            mpq_inv(tq2, f->value.mpq);
-            if (mpz_cmpabs(t2, t1) == 0 && mpq_cmp(tq1, tq2)==0) exp++;
-            if (mpz_cmpabs(t2, t1) > 0) exp++;  /* correct exponent */
-            mpq_clear(tq1);
-            mpq_clear(tq2);
-          }
+	  if (mpz_cmpabs(mpq_numref(f->value.mpq), mpq_denref(f->value.mpq)) >= 0)
+	  { mpz_tdiv_q(t1, mpq_numref(f->value.mpq), mpq_denref(f->value.mpq));
+	    exp = mpz_sizeinbase(t1, 10)-1;     /* guess exponent */
+	    mpz_ui_pow_ui(t2, 10, exp);
+	    if (mpz_cmpabs(t2, t1) > 0) exp--;  /* correct exponent */
+	  } else
+	  { mpq_t tq1, tq2;
+	    mpq_init(tq1);
+	    mpq_init(tq2);
+	    mpz_set(t1, mpq_numref(f->value.mpq));
+	    mpz_abs(t1, t1);
+	    mpz_tdiv_q(t1, mpq_denref(f->value.mpq), t1);
+	    exp = 0-mpz_sizeinbase(t1, 10);     /* guess exponent */
+	    mpz_ui_pow_ui(t2, 10, abs(exp)-1);
+	    mpq_set_z(tq1, t1);
+	    mpq_inv(tq2, f->value.mpq);
+	    if (mpz_cmpabs(t2, t1) == 0 && mpq_cmp(tq1, tq2)==0) exp++;
+	    if (mpz_cmpabs(t2, t1) > 0) exp++;  /* correct exponent */
+	    mpq_clear(tq1);
+	    mpq_clear(tq2);
+	  }
 
-          mpz_ui_pow_ui(t1, 10, abs(arg-exp));
-          if (mpz_cmpabs(mpq_numref(f->value.mpq), mpq_denref(f->value.mpq)) >= 0 && exp > arg)
-          { mpz_tdiv_q(t1, mpq_numref(f->value.mpq), t1);
-          } else
-          { mpz_mul(t1, mpq_numref(f->value.mpq), t1);
-          }
-          mpz_tdiv_q_ui(t2, mpq_denref(f->value.mpq), 2);
-          if (mpq_cmp_ui(f->value.mpq, 0, 1) < 0)
-          { mpz_sub(t1, t1, t2);
-            neg=1;
-          } else
-          { mpz_add(t1, t1, t2);
-            neg=0;
-          }
-          mpz_tdiv_q(t1, t1, mpq_denref(f->value.mpq));
-          mpz_abs(t1, t1);
+	  mpz_ui_pow_ui(t1, 10, abs(arg-exp));
+	  if (mpz_cmpabs(mpq_numref(f->value.mpq), mpq_denref(f->value.mpq)) >= 0 && exp > arg)
+	  { mpz_tdiv_q(t1, mpq_numref(f->value.mpq), t1);
+	  } else
+	  { mpz_mul(t1, mpq_numref(f->value.mpq), t1);
+	  }
+	  mpz_tdiv_q_ui(t2, mpq_denref(f->value.mpq), 2);
+	  if ( mpq_sgn(f->value.mpq) < 0)
+	  { mpz_sub(t1, t1, t2);
+	    neg=1;
+	  } else
+	  { mpz_add(t1, t1, t2);
+	    neg=0;
+	  }
+	  mpz_tdiv_q(t1, t1, mpq_denref(f->value.mpq));
+	  mpz_abs(t1, t1);
 
-          mpz_ui_pow_ui(t2, 10, arg+1);
-          if (mpz_cmpabs(t2, t1) == 0)
-          { mpz_tdiv_q_ui(t1, t1, 10);
-            exp++;
-          }
-          mpz_set_ui(t2, abs(exp));
+	  mpz_ui_pow_ui(t2, 10, arg+1);
+	  if (mpz_cmpabs(t2, t1) == 0)
+	  { mpz_tdiv_q_ui(t1, t1, 10);
+	    exp++;
+	  }
+	  mpz_set_ui(t2, abs(exp));
 
-        print_mpz_e:
+	print_mpz_e:
 
-          if (mpz_cmp_ui(t1, 0) == 0)
-            size = arg+7; /* reserve for 0.+e00<null> */
-          else
-            size = mpz_sizeinbase(t1, 10) + mpz_sizeinbase(t2, 10) + 6; /* reserve for -.e+0<null> */
+	  if ( mpz_sgn(t1) == 0 )
+	    size = arg+7; /* reserve for 0.+e00<null> */
+	  else
+	    size = mpz_sizeinbase(t1, 10) + mpz_sizeinbase(t2, 10) + 6; /* reserve for -.e+0<null> */
 
-          if ( !growBuffer(out, size) )
-          { PL_no_memory();
-            return NULL;
-          }
-          written = gmp_snprintf(baseBuffer(out, char), size, "%Zd%c%+03d", t1, how, exp);
+	  if ( !growBuffer(out, size) )
+	  { PL_no_memory();
+	    return NULL;
+	  }
+	  written = gmp_snprintf(baseBuffer(out, char), size, "%Zd%c%+03d", t1, how, exp);
 
-          if (mpz_cmp_ui(t1, 0) == 0)
-          { memmove(out->base+arg, out->base, written+1);
-            memset(out->base, '0', arg);
-            written += arg;
-          }
+	  if ( mpz_sgn(t1) == 0 )
+	  { memmove(out->base+arg, out->base, written+1);
+	    memset(out->base, '0', arg);
+	    written += arg;
+	  }
 
-          if (arg)
-          { memmove(out->base+2, out->base+1, written+1);
-            if ( locale->decimal_point && locale->decimal_point[0] )
-              *(out->base+1) = locale->decimal_point[0];
-            else
-              *(out->base+1) = '.';
-            written++;
-          }
+	  if (arg)
+	  { memmove(out->base+2, out->base+1, written+1);
+	    if ( locale->decimal_point && locale->decimal_point[0] )
+	      *(out->base+1) = (char)locale->decimal_point[0];
+	    else
+	      *(out->base+1) = '.';
+	    written++;
+	  }
 
-          if (neg)
-          { memmove(out->base+1, out->base, written+1);
-            memset(out->base, '-', 1);
-            written++;
-          }
+	  if (neg)
+	  { memmove(out->base+1, out->base, written+1);
+	    memset(out->base, '-', 1);
+	    written++;
+	  }
 
-          out->top = out->base + written;
-          mpz_clear(t1);
-          mpz_clear(t2);
-          return baseBuffer(out, char);
-        }
-        case 'g':
+	  out->top = out->base + written;
+	  mpz_clear(t1);
+	  mpz_clear(t2);
+	  return baseBuffer(out, char);
+	}
+	case 'g':
 	case 'G':
-        { switch(how)
-          { case 'g':
-            case 'G':
-            { mpz_t iv;
-              mpz_init(iv);
-              mpz_set_q(iv, f->value.mpq);
-              fbits = (int)mpz_sizeinbase(iv, 2) + 4*arg;
-              mpz_clear(iv);
-              break;
-            }
-            default:
-              fbits = 4*arg;
-          }
-          mpf_init2(mpf, fbits);
-          mpf_set_q(mpf, f->value.mpq);
+	{ switch(how)
+	  { case 'g':
+	    case 'G':
+	    { mpz_t iv;
+	      mpz_init(iv);
+	      mpz_set_q(iv, f->value.mpq);
+	      fbits = (int)mpz_sizeinbase(iv, 2) + 4*arg;
+	      mpz_clear(iv);
+	      break;
+	    }
+	    default:
+	      fbits = 4*arg;
+	  }
+	  mpf_init2(mpf, fbits);
+	  mpf_set_q(mpf, f->value.mpq);
 
-        print_mpf:
-          Ssprintf(tmp, "%%.%dF%c", arg, how);
-          size = 0;
-          written = arg+4;
-          while(written >= size)
-          { size = written+1;
+	print_mpf:
+	  Ssprintf(tmp, "%%.%dF%c", arg, how);
+	  size = 0;
+	  written = arg+4;
+	  while(written >= size)
+	  { size = written+1;
 
-            if ( !growBuffer(out, size) )	/* reserve for -.e<null> */
-            { PL_no_memory();
-              return NULL;
-            }
-            written = gmp_snprintf(baseBuffer(out, char), size, tmp, mpf);
-          }
-          mpf_clear(mpf);
-          out->top = out->base + written;
+	    if ( !growBuffer(out, size) )	/* reserve for -.e<null> */
+	    { PL_no_memory();
+	      return NULL;
+	    }
+	    written = gmp_snprintf(baseBuffer(out, char), size, tmp, mpf);
+	  }
+	  mpf_clear(mpf);
+	  out->top = out->base + written;
 
-          break;
-        }
+	  break;
+	}
       }
       break;
     }
-#endif
+#elif O_BF
+  { mpz_t n;
+
+    case V_MPZ:
+      mpz_init(n);
+      mpz_set(n, f->value.mpz);
+      goto bf_print;
+    case V_MPQ:
+    { bf_flags_t flags;
+      int upcase;
+      limb_t prec = ((double)(arg+2) * log(10)/log(2));
+
+      if ( how == 'f' )		/* we must compensate for the integer */
+      { mpz_t i;
+	mpz_init(i);
+	mpz_set_q(i, f->value.mpq);
+	prec += mpz_sizeinbase(i, 2);
+	mpz_clear(i);
+      }
+      mpz_init(n);
+      bf_div(n, mpq_numref(f->value.mpq), mpq_denref(f->value.mpq), prec, BF_RNDN);
+    bf_print:
+      upcase = false;
+      switch(how)
+      { case 'f':
+	  flags = BF_FTOA_FORMAT_FRAC;
+	  break;
+	case 'E':
+	  upcase = true;
+	case 'e':
+	  arg++;		/* LibBF counts total, we after . */
+	  flags = BF_FTOA_FORCE_EXP|BF_FTOA_FORMAT_FIXED;
+	  break;
+	case 'G':
+	  upcase = true;
+	case 'g':
+	  flags = BF_FTOA_FORMAT_FREE;
+	  break;
+	default:
+	  flags = 0;
+	  assert(0);
+      }
+      size_t size;
+      char *s = bf_ftoa(&size, n, 10, arg, BF_RNDN|BF_FTOA_PL_QUIRKS|flags);
+      if ( !growBuffer(out, size+1) )
+      { PL_no_memory();
+	return NULL;
+      }
+      strcpy(baseBuffer(out, char), s);
+      bf_free(n->ctx, s);
+      if ( upcase )
+      { char *exp = strchr(baseBuffer(out, char), 'e');
+	if ( exp )
+	  *exp = 'E';
+      }
+      mpz_clear(n);
+      break;
+    }
+  }
+#endif /* O_GMP OR O_BF */
     case V_INTEGER:
       promoteToFloatNumber(f);
       /*FALLTHROUGH*/
@@ -1631,30 +1715,30 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
 	}
 	written = snprintf(baseBuffer(out, char), size, tmp, f->value.f);
 #ifdef __WINDOWS__
-        if ( written < 0 )	/* pre-C99 Windows snprintf() returns -1 */
+	if ( written < 0 )	/* pre-C99 Windows snprintf() returns -1 */
 	  written = size*2;
 #endif
       }
 
 #ifdef __WINDOWS__
       /*
-         Write all ~e formatted floats in POSIX notation:
-         * exponent must contain at least 2 digits
-         * only as many digits as necessary to represent the exponent
+	 Write all ~e formatted floats in POSIX notation:
+	 * exponent must contain at least 2 digits
+	 * only as many digits as necessary to represent the exponent
       */
       switch(how)
       { case 'e':
-        case 'E':
-        case 'g':
-        case 'G':
-        { if (written >= 7 &&
-              ( *(out->base + written - 5) == 'e' ||
-                *(out->base + written - 5) == 'E' ) &&
-              ( *(out->base + written - 3) == '0' ))
-          { memmove(out->base + written-3, out->base + written-2, 3);
-            written--;
-          }
-        }
+	case 'E':
+	case 'g':
+	case 'G':
+	{ if (written >= 7 &&
+	      ( *(out->base + written - 5) == 'e' ||
+		*(out->base + written - 5) == 'E' ) &&
+	      ( *(out->base + written - 3) == '0' ))
+	  { memmove(out->base + written-3, out->base + written-2, 3);
+	    written--;
+	  }
+	}
       }
 #endif
 
@@ -1675,3 +1759,18 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
 
   return baseBuffer(out, char);
 }
+
+
+		 /*******************************
+		 *      PUBLISH PREDICATES	*
+		 *******************************/
+
+#define META PL_FA_TRANSPARENT
+#define NDET PL_FA_NONDETERMINISTIC
+
+BeginPredDefs(format)
+  PRED_DEF("format", 2, format, META)
+  PRED_DEF("format", 3, format, META)
+  PRED_DEF("format_predicate", 2, format_predicate, META)
+  PRED_DEF("current_format_predicate", 2, current_format_predicate, NDET|META)
+EndPredDefs
